using Distributions
using DataFrames

const n = 15
const dim = n+1

type GSM
  invC::Matrix{Float64}
  AᵀA::Matrix{Float64}
  sqrt_Q::Matrix{Float64}
  Aᵀx::Vector{Float64}
  A⁺x::Vector{Float64}
  σₓ²::Float64
  u_true::Vector{Float64}
  z_true::Float64
  frozen_noise::Vector{Float64}
end

off_diag_coef = 0.02
n_samples = 5000.

GSM() = begin
  #m = eye(n) + off_diag_coef*rand(n, n)
  #invC = m*m'
  m = off_diag_coef*rand(n, n)
  #invC = eye(n) + m + m'
  A_ = A()
  σₓ² = 0.1
  AᵀA = A_'*A_
  AᵀA = AᵀA/maximum(eig(AᵀA)[1])
  sqrt_Q = eye(dim) 
  sqrt_Q[1:n, 1:n] = sqrtm(inv(AᵀA))
  invC = AᵀA/(1-σₓ²)#rand(Wishart(n_samples, eye(n)/n_samples))
  C = full(inv(PDMats.PDMat(invC)))
  GSM(invC, AᵀA, sqrt_Q, zeros(n), zeros(n), σₓ², rand(MvNormal(zeros(n), C)), 0., 
                                     rand(MvNormal(zeros(n), σₓ²*AᵀA)))
end

gabor(n::Int, x0::Vector{Float64}, mag_k::Float64, theta::Float64, width::Float64) = begin
    k = mag_k * [sin(theta), cos(theta)]
    k_bar = [sin(theta), cos(theta)]
    k_perp = [-cos(theta), sin(theta)]
    aspect_ratio = 1 + 4*rand()
    #aspect_ratio = 1 + 10*rand()
    #aspect_ratio = 1 + rand()
    #Float64[(x = [i, j]/n - x0; exp(-dot(x, x)/(2*width^2))*sin(dot(x, k))) for i = 1:n, j=1:n]
    A = Float64[(x = [i, j]/n - x0; exp(-(dot(x, k_bar)^2 + (dot(x, k_perp)/aspect_ratio)^2)/(2*width^2))*sin(dot(x, k))) for i = 1:n, j=1:n]
end

A(;n::Int=100, mag_k::Float64=25., width::Float64=0.1) = begin
    result = zeros(n*n, 15)
    i = 0
    for x0 in Vector{Float64}[[1/6, 1/6], [1/6, 5/6], [5/6, 1/6], [5/6, 5/6], [1/2, 1/2]]
        for theta in [0., π/3, 2π/3]
            i += 1
            result[:, i] = reshape(gabor(n, x0, mag_k, theta, width), n*n)
        end
    end
    #result/sqrt(mean(diag(result'*result)))
    result/diagm(sqrt(diag(result'*result)))
end

dfA(A::Matrix{Float64}) = begin
    dfs = DataFrame[]
    for location = 1:5, angle=1:3
        push!(dfs, DataFrame(filter=A[:, (location-1)*3 + angle], 
                             x=repeat(103*(location-1)+collect(1:100); inner=[100]), 
                             y=repeat(103*(angle-1)+collect(1:100); outer=[100])))
    end
    vcat(dfs...)
end

writetable("filters", dfA(A()))

regen_contrast(g::GSM, z::Number) = begin
  #g.ATx = z*g.AᵀA*rand(MvNormalCannon(zeros(n), g.invC)) + rand(MvNormal(zeros(n), σₓ²*AᵀA))
  g.Aᵀx = z*g.AᵀA*g.u_true + g.frozen_noise
  g.A⁺x = g.AᵀA \ g.Aᵀx
  g.z_true = z
  Void()
end
