source("~/Dropbox/libs/Rtikz.R")
library(ggplot2)
library(gridExtra)

#linesize=0.5

#them <- theme_classic() +
#  theme(plot.title=element_text(hjust=-0.5, vjust=2.5)) +
#  #theme(plot.title=element_text(hjust=-0.3, vjust=1)) +
#  theme(line=element_line(size=linesize)) +
#  theme(axis.text=element_text(size=12)) +
#  theme(legend.key=element_blank())

l = 2500
plot <- function(filename, colour){
  data <- data.frame(read.csv(filename, colClasses=c("numeric", "character", "numeric")))
  #data    <- data.frame(t=1:l/10, u=datacsv[,1])
  data    <- subset(data, uv=="u")
  ggplot(data, aes(x=t, y=val)) +
  geom_line(size=linesize, colour=colour) +
  scale_colour_discrete("", labels=c("Excitatory ($u$)", "Inhibitory ($v$)")) +
  scale_x_continuous("$t$ (ms)", breaks=c(0, 100, 200), limits=c(0, 200)) +
  them
}

acf_ <- function(){
  lan <- data.frame(read.csv("langevin"))
  ham <- data.frame(read.csv("hamiltonian"))
  lan    <- subset(lan, uv=="u")
  ham    <- subset(ham, uv=="u")
  lan_acf <- acf(lan$val, lag.max=5000, plot=FALSE)
  ham_acf <- acf(ham$val, lag.max=5000, plot=FALSE)
  lan_acf_L <- acf(lan$L, lag.max=5000, plot=FALSE)
  ham_acf_L <- acf(ham$L, lag.max=5000, plot=FALSE)
  data <- rbind(
    data.frame(lag=ham_acf$lag/10, acf=ham_acf$acf, type="Hamiltonian"),
    data.frame(lag=lan_acf$lag/10, acf=lan_acf$acf, type="Langevin"))
  data_L <- rbind(
    data.frame(lag=ham_acf_L$lag/10, acf=ham_acf_L$acf, type="Hamiltonian"),
    data.frame(lag=lan_acf_L$lag/10, acf=lan_acf_L$acf, type="Langevin"))
  ggplot(data, aes(x=lag, y=acf, colour=type, group=type)) +
  geom_line(size=linesize) +
  geom_line(data=data_L, size=linesize, linetype="dashed") +
  scale_colour_manual("", values=c("red", "blue")) +
  scale_x_continuous("$t$ (ms)", limits=c(0, 150), expand=c(0,0), breaks=c(0, 100)) +
  scale_y_continuous("autocorrelation") +
  them + theme(legend.position="none")
}

p <- ggplot(data.frame(read.csv("comp_time", colClasses=c("numeric", "numeric", "numeric", "character"))), aes(x=t, y=mean, group=model, colour=model)) +
  geom_line(size=linesize) +
  geom_ribbon(aes(ymax=mean+sqrt(var)/5, ymin=mean-sqrt(var)/5, fill=model), colour=NA, alpha=0.4) +
  scale_x_continuous("$t$ (ms)", limits=c(0, 250), expand=c(0,0), breaks=c(0, 100, 200)) +
  scale_y_continuous(
    "MSE (a.u.)", 
    expand=c(0, 0),
    breaks=c(0, 5, 10)) +#, limits = c(0, 20)) +
  scale_colour_manual(
    "", 
    values=c("red", "blue"), 
    labels=c("Hamiltonian$\\;\\;\\;\\;\\;$", "Langevin")) +
  #scale_fill_discrete(guide="none") +
  scale_fill_manual(
    guide="none", 
    values=c("red", "blue")) +
  coord_cartesian(ylim=c(0, 14)) +
  them + theme(legend.position=c(0.7, 1.))


qlot <- function(filename){
  data <- data.frame(read.csv(filename, colClasses=c("numeric", "character", "numeric")))
  data <- subset(data, uv=="u" & 975<t & t<1000)
  psn <- data.frame(read.csv("psn", colClasses="numeric"))
  ggplot(data, aes(x=val, y=u2)) +
  geom_raster(data=psn, mapping=aes(x=u1, y=u2, fill=p)) +
  geom_path(size=linesize, mapping=aes(colour=t)) +
  #scale_colour_discrete("", labels=c("Excitatory (u)", "Inhibitory (v)")) +
  scale_fill_gradient(high="black", low="white") +
  scale_color_gradient(high="green", low="red") +
  scale_x_continuous("$u_1$ (a.u.)", limits=c(-1.5, 1.5), breaks=c(-1, 0, 1)) +
  scale_y_continuous("$u_2$ (a.u.)", limits=c(-1.5, 1.5), breaks=c(-1, 0, 1)) +
  them + theme(legend.position="none")
}

p1 <- plot("hamiltonian", "red") + theme(legend.position="none")#theme(legend.position=c(0.6, 1.1))
p2 <- plot("langevin", "blue") + theme(legend.position="none")

ymin <- function(plot) {
    ggplot_build(plot)$panel$ranges[[1]]$y.range[1]
}
ymax <- function(plot) {
    ggplot_build(plot)$panel$ranges[[1]]$y.range[2]
}
ymax <- max(ymax(p1), ymax(p2))
ymin <- min(ymin(p1), ymin(p2))

p1 <- p1 + scale_y_continuous("$u_1$ (a.u.)", limits=c(ymin, ymax))
p2 <- p2 + scale_y_continuous("$u_1$ (a.u.)", limits=c(ymin, ymax))

tikz("comparison.tex", width=pagewidth, height=2/3*pagewidth)
grid.arrange(
  #arrangeGrob(
    let(p1, "A"), 
    let(p2, "B"),
    let(acf_(), "C"),#+ theme(legend.position="none"), "C"), 
    let(qlot("hamiltonian"), "D"),
    let(qlot("langevin"), "E"), 
    let(p, "F"),
    ncol=3, nrow=2)#,
  #extract_legend(p + theme(legend.direction="horizontal")),
  #nrow=2, heights=c(1, 0.1))
dev.off()
