source("~/Dropbox/libs/Rtikz.R")
source("header.R")
library(ggplot2)
library(gridExtra)
library(plyr)
library(RColorBrewer)

myPalette <- colorRampPalette(rev(brewer.pal(11, "Spectral")))

them <- them + theme(plot.title = element_text(size=textsize, hjust=0.4))

#linesize=0.5
#
#them <- theme_classic() +
#  #theme(plot.title=element_text(hjust=-0.1)) +
#  theme(plot.title=element_text(hjust=-0.5, vjust=2.9)) +
#  theme(line=element_line(size=linesize)) +
#  #theme(text=element_text(size=15)) +
#  theme(axis.text=element_text(size=12)) +
#  theme(legend.text=element_text(size=13)) +
#  theme(legend.title=element_text(size=13)) +
#  theme(legend.key=element_blank()) +
#  theme(legend.title=element_text(face="plain"))

traces <- function(z1, letter) {
  ggplot(subset(data.frame(read.csv("contrast-traces", colClass="numeric")), z==z1), aes(t, u, group=run, color=factor(run))) +
  geom_line(size=linesize) +
  #scale_colour_brewer(palette="Set1") +
  scale_x_continuous("$t$ (ms)", limits=c(-200, 200), breaks=c(-200, 0, 200)) +
  scale_y_continuous("$u_i$", breaks=c(-4, 0, 4)) +
  geom_vline(xintercept=0, linetype="dotted", size=linesize) +
  scale_colour_brewer(type="seq", palette=2) +
  them + ggtitle(letter) +
  theme(legend.position="none")
}


#t_ffts
data <- data.frame(read.csv("contrast-tfft", colClass="numeric"))
lower_limit = 0.5
upper_limit = 4
data$power[data$power<lower_limit] = lower_limit
data$power[data$power>upper_limit] = upper_limit

tfft <- function(z1, letter, legend=FALSE) {
  ggplot(subset(data, z==z1), aes(x=t, y=freq, fill=power)) +
  scale_y_continuous("Freq. (Hz)", limits=c(0,200), breaks=c(0, 100, 200)) +
  scale_x_continuous("$t$ (ms)", breaks=c(-750, 0, 750)) +
  #scale_fill_gradient(high="black", low="white", limits=c(log(lower_limit), log(upper_limit))) +
  #scale_fill_gradient(high="black", low="white", limits=c(log(lower_limit), log(upper_limit))) +
  scale_fill_gradientn("\\rm{Power}", colours = myPalette(100), limits=c(lower_limit, upper_limit), trans = "log", breaks=c(0.5, 1, 2, 4)) +
  geom_raster() +
  geom_vline(xintercept=0, linetype="dotted", size=linesize) +
  them + theme(legend.key.height = unit(0.3, "cm"), legend.key.width = unit(0.3, "cm")) +
  if (legend){
    theme()}
  else{
  theme(legend.position="none")}
}

#ffts
fft_data <- data.frame(read.csv("contrast-fft", colClass="numeric"))
#fft_data <- ddply(fft_data, c("z"), transform, power = power/fft_data[fft_data$z==0, "power"])
#fft_data <- subset(fft_data, z != 0)

fft <- function(letter){
  #fft_data <- data.frame(read.csv(file, colClass="numeric"))
  #fft_data <- ddply(fft_data, c("z"), transform, power = power/fft_data[fft_data$z==0, "power"])
  fft <- ggplot(fft_data, aes(freq, power*freq, group=z, colour=factor(z))) +
  #Can add baseline to power of ~ 300/freq
    geom_line(size=linesize) +
    scale_y_log10(
      "Power $\\times$ Freq. (a.u.)", 
      breaks=c(100, 10000, 1000000), 
      labels=c("$10^2$", "$10^4$", "$10^6$")) +
    scale_x_continuous("Freq. (Hz)", limits=c(0, 200), breaks=c(0, 200)) +
    #scale_colour_discrete("$z_{\\rm{gen}}$", direction=-1) +
    #scale_colour_brewer("$z_{\\rm{gen}}$", palette=2, type="seq", values=) +
    scale_colour_manual("$z_{\\rm{gen}}$", values=cols4) +
    them + ggtitle(letter) +
    theme(legend.position="right")
}

#ffts

fft_data_n <- subset(fft_data, freq>4 & freq<100)
freq_data <- ddply(fft_data_n, .(z), summarise, peak_freq = freq[which.max(power*freq)])
#freq_data <- ddply(fft_data, .(z), summarise, peak_freq = sum((z!=2 | freq>20)*freq*power)/sum((z!=2 | freq>20)*power))
freq_data$predicted <- sqrt(1+10*freq_data$z^2)/(2*pi*0.01)
freq <- ggplot(freq_data, aes(y=peak_freq, x=predicted, colour=factor(z))) +
  geom_abline(size=linesize) +
  geom_point(size=3) +
  scale_y_continuous("Sim. freq. (Hz)", limits=c(0, 110), breaks=c(0, 50, 100)) +
  scale_x_continuous("Pred. freq. (Hz)", limits=c(0, 110), breaks=c(0, 50, 100)) +
  scale_colour_manual(values=cols4) +
  them +
  theme(legend.position="none") +
  theme(axis.title.x=element_text(size=textsize, hjust=0.8))


blankPanel<-grid.rect(gp=gpar(col="white"))
empty <- ggplot() + 
  geom_point(aes(1,1), colour="white")+
  theme(axis.ticks=element_blank(), 
        panel.background=element_blank(), 
        axis.text.x=element_blank(), axis.text.y=element_blank(),           
        axis.title.x=element_blank(), axis.title.y=element_blank(),
        panel.grid=element_blank())

#extract legend
  #https://github.com/hadley/ggplot2/wiki/Share-a-legend-between-two-ggplot2-graphs
g_legend<-function(a.gplot){
  tmp <- ggplot_gtable(ggplot_build(a.gplot))
  leg <- which(sapply(tmp$grobs, function(x) x$name) == "guide-box")
  legend <- tmp$grobs[[leg]]
  return(legend)}

mylegend<-g_legend(tfft(2., '', legend=TRUE))

    #traces(0.5, "\\textbf{A} $\\;\\;\\;\\;\\;\\; z_{\\text{gen}}=0.5$") + theme(plot.title=element_text(hjust=2)),
#Collect all
tikz("contrast.tex", width=4/3*pagewidth, height=2/3*pagewidth)
grid.arrange(
  arrangeGrob(
    let(traces(0.5,"$z_{\\rm{gen}}=0.5$"), "A"),
    let(traces(1., "$z_{\\rm{gen}}=1$"), ""),
    let(traces(2., "$z_{\\rm{gen}}=2$"), ""),
    #empty,
    let(fft(''), "B"),
    ncol=5, widths=c(1,1,1,1.5,0.01)),
  arrangeGrob(
    let(tfft(0.5, ''), "C"),
    let(tfft(1., ''), ""),
    let(tfft(2., ''), ""),
    arrangeGrob(mylegend, blankPanel, nrow=2, heights=c(1, 0.25)),
    let(freq, "D"),
    ncol=5, widths=c(1,1,1,0.5,1)),
  nrow=2)
dev.off()
