source("~/Dropbox/libs/Rtikz.R")

library(ggplot2)
library(gridExtra)
library(scales)

#them <- theme_classic() +
#  theme(plot.title=element_text(hjust=-0.1)) + 
#  theme(legend.key=element_blank())

#Collect all

sim <- data.frame(read.csv("hmc-demo-simulation"))
sim$E <- 1
arrows <- data.frame(read.csv("hmc-demo-arrows"))
arrows$E <- 1
land <- data.frame(read.csv("hmc-demo-landscape"))
new_max <- 0.15*max(land$E)
land$E[land$E>new_max] <- new_max



A <- 
  ggplot(land, aes(x=x1, y=x2, fill=E)) +
  geom_raster() +
  scale_fill_gradient(limits=c(min(land$E), new_max), low="black", high="white") +
  geom_segment(data=arrows, mapping=aes(xend=x1_end, yend=x2_end), size=0.5, color="red", arrow=arrow(length=unit(0.1, "cm"))) +
  labs(y="$x_2$ (a.u.)", x="$x_1$ (a.u.)", fill="Energy (a.u.)") +
  them + theme(legend.position="none")

d <- rbind(data.frame(x=sim$t, y=sim$x1, typ="Position"), data.frame(x=sim$t, y=sim$v1, typ="Momentum"))
B <- 
  ggplot(d, aes(x=x, y=y, colour=typ, group=typ)) +
  geom_hline(yintercept=0, colour="gray") +
  geom_line(size=linesize) +
  labs(y="$x_1$ or $v_1$ (a.u.)", x="Time (a.u.)", colour="") +
  scale_colour_discrete(labels = c("$x_1$", "$v_1$")) +
  them + 
  theme(
    legend.position=c(1.0, 1.0),
    legend.background=element_rect(fill="transparent")
    )

C <- 
  ggplot(arrows, aes(x=x1, y=v1)) +
  geom_hline(yintercept=0, colour="gray") +
  geom_vline(xintercept=0, colour="gray") +
  geom_segment(data=arrows, mapping=aes(xend=x1_end, yend=v1_end), size=0.5, color="red", arrow=arrow(length=unit(0.1, "cm"))) +
  labs(x="$x_1$ (a.u.)", y="$v_1$ (a.u.)") +
  scale_y_continuous(breaks=c(-0.5, 0, 0.5)) +
  them

D <- 
  ggplot(sim, aes(x=Es, y=Ks)) +
  geom_path() +
  labs(x="PE, $\\;-\\log P(\\mathbf{x})$", y="KE, $\\; |\\mathbf{v}|^2/2$") +
  them

A <- ggplot_gtable(ggplot_build(A))
B <- ggplot_gtable(ggplot_build(B))
C <- ggplot_gtable(ggplot_build(C))
D <- ggplot_gtable(ggplot_build(D))

maxWidth = unit.pmax(A$widths[2:3], C$widths[2:3])
A$widths[2:3] <- maxWidth
C$widths[2:3] <- maxWidth

maxWidth = unit.pmax(B$widths[2:3], D$widths[2:3])
B$widths[2:3] <- maxWidth
D$widths[2:3] <- maxWidth

A <- let(A, "A")
B <- let(B, "B")
C <- let(C, "C")
D <- let(D, "D")

tikz("hmc.tex", width=pagewidth, height=2/3*pagewidth)
grid.arrange(A, B, C, D, nrow=2, ncol=2)
dev.off()
