source("header.R")
source("~/Dropbox/libs/Rtikz.R")
library(ggplot2)
library(gridExtra)
library(plyr)

#linesize=0.5
#
#them <- theme_classic() +
#  #theme(plot.title=element_text(hjust=-0.1)) +
#  theme(plot.title=element_text(hjust=-0.8, vjust=2.9)) +
#  theme(line=element_line(size=linesize)) +
#  #theme(text=element_text(size=15)) +
#  theme(axis.text=element_text(size=12)) +
#  theme(legend.key=element_blank()) +
#  theme(legend.title=element_text(face="plain"))


fft_with_legend <- function(file, ylabel){
  ggplot(data.frame(read.csv(file, colClasses="numeric")), aes(freq, power*freq, group=z, colour=factor(z))) +
  geom_line(size=linesize) +
  scale_y_log10(
    "Power$\\times$Freq.\n (a.u.)",
    limits=c(10, 100000),
    breaks=c(100, 10000),
    labels=c("$10^2$", "$10^4$"))  +
  scale_x_continuous("Frequency (Hz)", limits=c(0, 200), breaks=c(0, 100, 200)) +
  scale_colour_manual("$z_{\\rm{gen}}$", values=cols3) +
  them + 
  if (ylabel) {
    theme()
  } else {
    theme(axis.title.y = element_blank())
  }
}
fft <- function(file, ylabel=FALSE){fft_with_legend(file, ylabel) + theme(legend.position="none")}

transient <- function(data, ylabel=FALSE){
  ggplot(data.frame(read.csv(data, colClasses="numeric")), aes(t, rate, group=z, colour=factor(z))) +
  geom_line(size=linesize) +
  geom_vline(xintercept=0, linetype="longdash", size=linesize) +
  scale_x_continuous("$t$ (ms)", limits=c(-100, 100), breaks=c(-100, 0, 100)) +
  scale_y_continuous("firing rate (a.u.)", limits=c(0, 10), breaks=c(0, 5, 10)) +
  scale_colour_manual("$z_{\\rm{gen}}$", values=cols3) +
  expand_limits(y=0) +
  them + theme(legend.position="none") +
  if (ylabel) {
    theme()
  } else {
    theme(axis.title.y = element_blank())
  }
}

trace <- function(file, ylabel=FALSE) {
  ggplot(data.frame(read.csv(file, colClasses="numeric")), aes(t, u, group=run, color=factor(run))) +
  geom_line(size=linesize) +
  scale_x_continuous("$t$ (ms)", limits=c(-100, 100), breaks=c(-100, 0, 100)) +
  scale_y_continuous("$u_i$ (a.u.)", limits=c(-4, 4), breaks=c(-4, 0, 4)) +
  geom_vline(xintercept=0, linetype="longdash", size=linesize) +
  scale_colour_brewer(type="seq", palette=2) +
  them + 
  theme(legend.position="none")+ 
  if (ylabel) {
    theme()
  } else {
    theme(axis.title.y = element_blank())
  }
}

g_legend<-function(a.gplot){
  tmp <- ggplot_gtable(ggplot_build(a.gplot))
  leg <- which(sapply(tmp$grobs, function(x) x$name) == "guide-box")
  legend <- tmp$grobs[[leg]]
  return(legend)}

tikz("supplementary.tex", width=4/3*pagewidth, height=5/6*pagewidth)

p1 <- ggplotGrob(fft("fft-tl=30", ylabel=TRUE))
p2 <- ggplotGrob(fft("fft-tl=60"))
p3 <- ggplotGrob(fft("fft-tl=400"))
p4 <- ggplotGrob(fft("fft-tl=1000"))
p5 <- ggplotGrob(transient("rate-tl=30", ylabel=TRUE))
p6 <- ggplotGrob(transient("rate-tl=60"))
p7 <- ggplotGrob(transient("rate-tl=400"))
p8 <- ggplotGrob(transient("rate-tl=1000"))
p9 <- ggplotGrob(trace("trace-tl=30", ylabel=TRUE))
pa <- ggplotGrob(trace("trace-tl=60"))
pb <- ggplotGrob(trace("trace-tl=400"))
pc <- ggplotGrob(trace("trace-tl=1000"))

maxWidth = grid::unit.pmax(p1$widths[2:3], p5$widths[2:3], p9$widths[2:3])
p1$widths[2:3] <- as.list(maxWidth)
p5$widths[2:3] <- as.list(maxWidth)
p9$widths[2:3] <- as.list(maxWidth)

maxWidth = grid::unit.pmax(p2$widths[2:3], p6$widths[2:3], pa$widths[2:3])
p2$widths[2:3] <- as.list(maxWidth)
p3$widths[2:3] <- as.list(maxWidth)
p4$widths[2:3] <- as.list(maxWidth)
p6$widths[2:3] <- as.list(maxWidth)
p7$widths[2:3] <- as.list(maxWidth)
p8$widths[2:3] <- as.list(maxWidth)
pa$widths[2:3] <- as.list(maxWidth)
pb$widths[2:3] <- as.list(maxWidth)
pc$widths[2:3] <- as.list(maxWidth)

grid.arrange(
  let(p1, "A"), 
  let(p2, "B"), 
  let(p3, "C"), 
  let(p4, "D"),  
  g_legend(fft_with_legend("fft-tl=30", TRUE)),
  p5, 
  p6, 
  p7, 
  p8, 
  blankPanel,
  p9, 
  pa, 
  pb, 
  pc, 
  blankPanel,
  nrow=3, ncol=5,
  widths=c(1.2, 1, 1, 1, 0.5),
  heights=c(1.2, 1, 1))
#grid.arrange(arrangeGrob(let(p1, "A"), let(p2, "B"), let(p3, "C"), let(p4, "D"),  p5, p6, p7, p8, p9, pa, pb, pc,#fft("fft-tl=30",   'A'), 
#                        #fft("fft-tl=60",   'B'), 
#                        #fft("fft-tl=400",  'C'), 
#                        #fft("fft-tl=1000", 'D'),
#                        #transient("rate-tl=30"),
#                        #transient("rate-tl=60"),
#                        #transient("rate-tl=400"),
#                        #transient("rate-tl=1000"),
#                         nrow=3, ncol=4),
#             g_legend(fft_with_legend("fft-tl=30", 'A')), ncol=2,widths=c(10, 1))
dev.off()
