source("header.R")
source("~/Dropbox/libs/Rtikz.R")
library(ggplot2)
library(gridExtra)
library(RColorBrewer)

#linesize=0.75

myPalette <- colorRampPalette(rev(brewer.pal(11, "Spectral")))

#them <- theme_classic() +
#  theme(plot.title=element_text(hjust=-0.3, vjust=2.5)) +
#  theme(line=element_line(size=linesize)) +
#  #theme(text=element_text(size=15)) +
#  theme(axis.text=element_text(size=12)) +
#  theme(legend.key=element_blank()) +
#  theme(legend.title=element_text(face="plain"))

rates <- function(data, letter, ymax){
  df <- data.frame(read.csv(data, colClasses="numeric"))
  df <- subset(df, z != 0.)
  ggplot(df, aes(t, rate, group=z, colour=factor(z))) +
  geom_line(size=linesize) +
  #geom_vline(x=0, linetype="dotted", size=linesize) +
  scale_x_continuous("$t$ (ms)", limits=c(-20, 100), breaks=c(0, 100)) +
  scale_y_continuous("firing rate (a.u.)", limits=c(0, ymax), breaks=c(0, ymax/2, ymax)) +
  #scale_colour_discrete(expression(z[gen])) +
  #scale_colour_discrete("$z_{\\rm{gen}}$") +
  scale_colour_manual("$z_{\\rm{gen}}$", values=cols3) +
  expand_limits(y=0) +
  them + theme(legend.position="none")
}

us <- seq(0, 5, by=0.01)
df <- rbind(data.frame(u=us, z=1, typ="$z$ fixed"),
            data.frame(u=us, z=1/us, typ="$z$ inferred"))
D <- ggplot(df, aes(x=u, y=z, colour=typ, group=typ)) +
    geom_line(size=linesize) + 
    scale_y_continuous("$z$", limits=c(0, 10), breaks=c(0, 5, 10)) +
    scale_x_continuous("$u$", limits=c(0, 5)) +
    scale_colour_manual("", values=c("red", "blue")) +
    them + theme(legend.position=c(0.7,0.8)) +
    theme(axis.title.y = element_text(angle=00))
D <- let(D, "D")
#p <- ggplot(data.frame(read.csv("transient-spring", colClasses="numeric")), aes(x=u, y=s, colour=factor(z), group=factor(z))) +
#    geom_line(size=linesize) + 
#    scale_y_continuous("k(u) (a.u.)", limits=c(0, 10)) +
#    scale_x_continuous(limits=c(0, 5)) +
#    scale_colour_discrete(expression(z[gen])) +
#    ggtitle('E') +
#    them #+ theme(legend.position="none")

us <- seq(0, 5, by=0.01)
df <- rbind(data.frame(u=us, udd=-(us-1), typ="$z$ fixed"),
            data.frame(u=us, udd=-(us-1)*(1+1/us^2), typ="$z$ inferred"))
E <- ggplot(df, aes(x=u, y=udd, colour=typ, group=typ)) +
    geom_line(size=linesize) + 
    scale_y_continuous("$\\frac{d^2u}{dt^2}$", limits=c(-5, 10)) +
    scale_x_continuous("$u$", limits=c(0, 5)) +
    scale_colour_manual("", values=c("red", "blue")) +
    #geom_hline(linetype="dotted", size=linesize, y=0) +
    them + theme(legend.position="none") +
    theme(axis.title.y = element_text(angle=00))
E <- let(E, "E")
    
    #theme(legend.position=c(0.7, 0.5))#+ theme(legend.position="none")

F_ <- ggplot(data.frame(read.csv("transient-sim", colClasses="numeric")), aes(x=x, y=y, colour=factor(z), group=factor(z))) +
    geom_line(size=linesize) + 
    #geom_vline(x=0, linetype="dotted", size=linesize) +
    #scale_colour_discrete("$z_{gen}$") +
    scale_colour_manual("$z_{gen}$", values=cols3) +
    scale_x_continuous("$t$ (ms)", limits=c(0, 32), breaks=c(0, 100)) +
    coord_cartesian(xlim=c(-30, 110)) +
    scale_y_continuous("$u$", breaks=c(0, 5, 10)) +
    them + theme(legend.position="none") +
    theme(axis.title.y = element_text(angle=00))
F_ <- let(F_, "F")


tikz("transient.tex", width=pagewidth, height=2/3*pagewidth)
grid.arrange(let(rates("contrast-rate", '', 20), "A"),
             let(rates("contrast-rate-langevin", '', 6), "B"), 
             let(rates("contrast-rate-shm", "", 6)+ theme(legend.position=c(0.85, 0.7)), "C"), D, E, F_, 
             nrow=2, ncol=3)
dev.off()
